@@ -2040,7 +2040,7 @@ float UDrawRescaleBox(XPoint *corner)
 	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
 	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
 	 }
-	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 else if (newscale < 0.1 * rlab->scale) newscale = 0.1 * rlab->scale;
 	 rlab->scale = newscale;
 	 labelbbox(rlab, newpoints, areawin->topinstance);
 	 rlab->scale = savescale;
@@ -2100,7 +2100,7 @@ float UDrawRescaleBox(XPoint *corner)
 	    newscale = (float)((int)(newscale * snapstep)) / snapstep;
 	    if (newscale < (1.0 / snapstep)) newscale = (1.0 / snapstep);
 	 }
-	 else if (newscale < 0.1 * rgraph->scale) newscale = 0.1 * rgraph->scale;
+	 else if (newscale < 0.1 * rinst->scale) newscale = 0.1 * rinst->scale;
 	 rinst->scale = newscale;
 	 objinstbbox(rinst, newpoints, 0);
 	 rinst->scale = savescale;
@@ -2401,9 +2401,12 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 	     break;
    
   	  case(LABEL): 
-	     if (level == 0 || TOLABEL(areagen)->pin == False ||
-			(TOLABEL(areagen)->justify & PINVISIBLE))
-             UDrawString(TOLABEL(areagen), curcolor, theinstance);
+	     if (level == 0 || TOLABEL(areagen)->pin == False)
+                UDrawString(TOLABEL(areagen), curcolor, theinstance);
+	     else if ((TOLABEL(areagen)->justify & PINVISIBLE) && areawin->pinpointon)
+                UDrawString(TOLABEL(areagen), curcolor, theinstance);
+	     else if (TOLABEL(areagen)->justify & PINVISIBLE)
+                UDrawStringNoX(TOLABEL(areagen), curcolor, theinstance);
 	     else if (level == 1 && TOLABEL(areagen)->pin &&
 			TOLABEL(areagen)->pin != INFO && areawin->pinpointon)
 		UDrawXDown(TOLABEL(areagen));
@@ -2525,6 +2528,7 @@ void TopDoLatex()
    float psscale, outscale;
    int tx, ty, width, height;
    XPoint origin;
+   polyptr framebox;
    Boolean checklatex = FALSE;
    char filename[100], extend[10], *dotptr;
 
@@ -2572,12 +2576,31 @@ void TopDoLatex()
    width = toplevelwidth(areawin->topinstance, &origin.x);
    height = toplevelheight(areawin->topinstance, &origin.y);
 
+   /* Added 10/19/10:  If there is a specified bounding box, let it	*/
+   /* determine the figure origin;  otherwise, the labels will be	*/
+   /* mismatched to the bounding box.					*/
+
+   if ((framebox = checkforbbox(topobject)) != NULL) {
+      int i, maxx, maxy;
+
+      origin.x = maxx = framebox->points[0].x;
+      origin.y = maxy = framebox->points[0].y;
+      for (i = 1; i < framebox->number; i++) {
+	 if (framebox->points[i].x < origin.x) origin.x = framebox->points[i].x;
+	 if (framebox->points[i].x > maxx) maxx = framebox->points[i].x;
+	 if (framebox->points[i].y < origin.y) origin.y = framebox->points[i].y;
+	 if (framebox->points[i].y > maxy) maxy = framebox->points[i].y;
+      }
+      origin.x -= ((width - maxx + origin.x) / 2);
+      origin.y -= ((height - maxy + origin.y) / 2);
+   }
+
    tx = (int)(72 / psscale) - origin.x,
    ty = (int)(72 / psscale) - origin.y;
 
-   fprintf(f, "   \\scalebox{%g}{%\n", outscale);
+   fprintf(f, "   \\scalebox{%g}{\n", outscale);
    fprintf(f, "   \\normalsize\n");
-   fprintf(f, "   \\parbox{%gin}{%\n", (((float)width * psscale) / 72.0) / outscale);
+   fprintf(f, "   \\parbox{%gin}{\n", (((float)width * psscale) / 72.0) / outscale);
    fprintf(f, "   \\includegraphics[scale=%g]{%s%s}\\\\\n", 1.0 / outscale,
 			filename, extend);
    fprintf(f, "   %% translate x=%d y=%d scale %3.2f\n", tx, ty, psscale);
