@@ -5,6 +5,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <math.h>
 #include <limits.h>
@@ -1061,6 +1062,8 @@ OutputSVG(char *filename, Boolean fullscale)
    UPopCTM();	/* Restore the top-level graphics state */
 }
 
+#ifdef TCL_WRAPPER
+
 /*----------------------------------------------------------------------*/
 /* The TCL command-line for the SVG file write routine.			*/
 /*----------------------------------------------------------------------*/
@@ -1108,4 +1111,6 @@ int xctcl_svg(ClientData clientData, Tcl_Interp *interp,
    return XcTagCallback(interp, objc, objv);
 }
 
+#endif /* TCL_WRAPPER */
+
 /*-------------------------------------------------------------------------*/
