@@ -199,7 +199,7 @@ stringpart *splitstring(int tpos, stringpart **strtop, objinstptr localinst)
 }
 
 /*----------------------------------------------------------------------*/
-/* Get the next string part, linking to a paramter if necessary		*/
+/* Get the next string part, linking to a parameter if necessary	*/
 /*----------------------------------------------------------------------*/
 
 stringpart *nextstringpartrecompute(stringpart *strptr, objinstptr thisinst)
@@ -372,7 +372,8 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart,
 
    if (localinst == NULL) {
       ops = match_param(topobject, key);
-      if (ops == NULL) return NULL;
+      if (ops == NULL)
+	 return NULL;
    }
    else {
       ops = find_param(localinst, key);
@@ -457,7 +458,8 @@ int findcurfont(int tpos, stringpart *strtop, objinstptr thisinst)
    stringpart *strptr;
 
    curpos = findstringpart(tpos, NULL, strtop, thisinst);
-   for (strptr = strtop; strptr != curpos; strptr = nextstringpart(strptr, thisinst))
+   for (strptr = strtop; (strptr != NULL) && (strptr != curpos);
+		strptr = nextstringpart(strptr, thisinst))
       if (strptr->type == FONT_NAME)
          cfont = strptr->data.font;
 
@@ -817,6 +819,19 @@ Boolean issymbolfont(int fontnumber)
    return False;
 }
 
+/*----------------------------------------------------------------------*/
+/* Test if the specified font is in the "Helvetica" font family.	*/
+/* SVG uses this to make sure it scales down the font by 7/8 to match	*/
+/* our internal vectors, and to use "oblique" for the style instead of	*/
+/* "italic".								*/
+/*----------------------------------------------------------------------*/
+
+Boolean issansfont(int fontnumber)
+{
+   if (!strcmp(fonts[fontnumber].family, "Helvetica")) return True;
+   return False;
+}
+
 /*----------------------------------------------------------------------*/
 /* Test if the specified font is ISO-Latin1 encoding			*/
 /*----------------------------------------------------------------------*/
@@ -1363,9 +1378,11 @@ short UDrawChar(u_char code, short styles, short ffont, int groupheight, int pas
 
 /*----------------------------------------------------------------------*/
 /* Draw an entire string, including parameter substitutions		*/
+/* (Normally called as UDrawString(); see below)			*/
 /*----------------------------------------------------------------------*/
 
-void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
+void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
+	Boolean drawX)
 {
    stringpart *strptr;
    char *textptr;
@@ -1592,7 +1609,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 		   char *nsptr = strstr(textptr, "::");
 		   if (nsptr != NULL) {
 		      textptr = nsptr + 2;
-		      pos += (int)nsptr - (int)strptr->data.string + 2;
+		      pos += (pointertype)nsptr - (pointertype)strptr->data.string + 2;
 		   }
 		}
 
@@ -1632,7 +1649,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 
    UPopCTM();
 
-   if (drawlabel->pin) UDrawXDown(drawlabel);
+   if (drawX && drawlabel->pin) UDrawXDown(drawlabel);
 
    if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
       XTopSetForeground(passcolor);
@@ -1641,6 +1658,24 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    xobjs.pagelist[areawin->page]->wirewidth = tmpthick;
 }
 
+/*----------------------------------------------------------------------*/
+/* Draw String, with an "x" mark at the origin				*/
+/*----------------------------------------------------------------------*/
+
+void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
+{
+   UDrawString0(drawlabel, passcolor, localinst, TRUE);
+}
+
+/*----------------------------------------------------------------------*/
+/* Draw String, without the "x" mark					*/
+/*----------------------------------------------------------------------*/
+
+void UDrawStringNoX(labelptr drawlabel, int passcolor, objinstptr localinst)
+{
+   UDrawString0(drawlabel, passcolor, localinst, FALSE);
+}
+
 /*----------------------------------------------------------------------*/
 /* Compute the actual length of a string or portion thereof.		*/
 /*----------------------------------------------------------------------*/
@@ -1763,7 +1798,7 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 		char *nsptr = strstr(textptr, "::");
 		if (nsptr != NULL) {
 		   textptr = nsptr + 2;
-		   locpos += (int)nsptr - (int)strptr->data.string + 2;
+		   locpos += (pointertype)nsptr - (pointertype)strptr->data.string + 2;
 		}
 	    }
 
