@@ -85,6 +85,8 @@ typedef struct {
    float scale;			/* old scale value */
 } scaleinfo;
 
+Boolean undo_collect = FALSE;
+
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
 /*----------------------------------------------------------------------*/
@@ -144,7 +146,7 @@ uselection *remember_selection(objinstptr topinst, short *slist, int number)
 }
 
 /*----------------------------------------------------------------------*/
-/* Create a selection list in areawin from the saved uselection	*/
+/* Create a selection list in areawin from the saved uselection		*/
 /* record.								*/
 /*----------------------------------------------------------------------*/
 
@@ -372,7 +374,9 @@ void undelete_one_element(objinstptr thisinst, genericptr thiselem)
 /*									*/
 /*   "mode" is UNDO_MORE if one or more undo records are expected to 	*/
 /*   follow in a series, or UNDO_DONE if this completes a series, or is	*/
-/*   as single undo event.						*/
+/*   as single undo event. The command-line command "undo series start"	*/
+/*   forces all events to be UNDO_MORE until "undo series end" is	*/
+/*   issued.								*/
 /*----------------------------------------------------------------------*/
 
 void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
@@ -418,7 +422,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    else
       newrecord->idx = 1;
 
-   if (mode == UNDO_MORE)
+   if (mode == UNDO_MORE || undo_collect == TRUE)
       newrecord->idx = -newrecord->idx;
    
    xobjs.undostack = newrecord;
@@ -481,7 +485,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 /* 1 arg:							*/
 	 /*	egen = element just created (genericptr)		*/
 	 egen = va_arg(args, genericptr);
@@ -738,7 +742,7 @@ short undo_one_action()
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
 		case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-		case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+		case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 egen = (genericptr)thisrecord->undodata;
 	 i = thisrecord->thisinst->thisobject->parts - 1;
 	 if (egen != *(thisrecord->thisinst->thisobject->plist + i)) {
@@ -985,6 +989,21 @@ short undo_one_action()
    return thisrecord->idx;
 }
 
+/*----------------------------------------------------------------------*/
+/* undo_finish_series ---						*/
+/*	Complete a possibly incomplete undo series by forcing the	*/
+/*	topmost entry to have a positive index.				*/
+/*	Note that for "undo series start|end" to work, undo_collect	*/
+/*	must be set to FALSE prior to calling undo_finish_series().	*/
+/*----------------------------------------------------------------------*/
+
+void undo_finish_series()
+{
+   if (undo_collect == FALSE)
+      if (xobjs.undostack && xobjs.undostack->idx < 0)
+         xobjs.undostack->idx = -xobjs.undostack->idx;
+}
+
 /*----------------------------------------------------------------------*/
 /* undo_action ---							*/
 /*	Play undo record back to the completion of a series.		*/
@@ -1120,7 +1139,7 @@ short redo_one_action()
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 egen = (genericptr)thisrecord->undodata;
 	 undelete_one_element(thisrecord->thisinst, egen);
 	 drawarea(areawin->area, NULL, NULL);
@@ -1427,7 +1446,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 /* if MODE_UNDO, the element is on the page, so don't destroy it! */
 	 if (mode == MODE_REDO)
 	    free(thisrecord->undodata);
