@@ -807,7 +807,8 @@ void setfilename(xcWidget w, char **dataptr)
    objectptr checkpage;
    char *oldstr = xobjs.pagelist[areawin->page]->filename;
 
-   if (!strcmp(*dataptr, _STR2)) return;   /* no change in string */
+   if ((*dataptr != NULL) && !strcmp(*dataptr, _STR2))
+      return;   /* no change in string */
 
    /* Make the change to the current page */
    xobjs.pagelist[areawin->page]->filename = strdup(_STR2);
@@ -815,7 +816,7 @@ void setfilename(xcWidget w, char **dataptr)
    /* All existing filenames which match the old string should also be changed */
    for (cpage = 0; cpage < xobjs.pages; cpage++) {
       if ((xobjs.pagelist[cpage]->pageinst != NULL) && (cpage != areawin->page)) {
-	 if (!strcmp(xobjs.pagelist[cpage]->filename, oldstr)) {
+	 if ((oldstr != NULL) && (!strcmp(xobjs.pagelist[cpage]->filename, oldstr))) {
 	    free(xobjs.pagelist[cpage]->filename);
 	    xobjs.pagelist[cpage]->filename = strdup(_STR2);
 	 }
@@ -1213,8 +1214,10 @@ void togglegrid(u_short type)
    else if (type == DEC_INCH) button = XtNameToWidget(bparent, "Decimal Inches");
    else if (type == INTERNAL) button = XtNameToWidget(bparent, "Internal Units");
    else button = XtNameToWidget(bparent, "Coordinates");
-   toggleexcl(button, GridStyles, XtNumber(GridStyles));
-   W1printf(" ");
+   if (button) {
+      toggleexcl(button, GridStyles, XtNumber(GridStyles));
+      W1printf(" ");
+   }
 }
 
 /*----------------------------------------------------------------*/
@@ -1562,7 +1565,7 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
 	    redrawtext(curlabel);
       }
       else {
-	 sprintf(_STR2, "%d", cindex);
+	 // sprintf(_STR2, "%d", cindex);
 	 labeltext(FONT_COLOR, (char *)&cindex);
       }
    }
